<?php
namespace Drupal\content_export_yaml;

use Drupal\node\Entity\Node;
use Symfony\Component\Yaml\Dumper;
use Symfony\Component\Yaml\Yaml as SymfonyYaml;
use Symfony\Component\Yaml\Parser;
/**
 * Created by PhpStorm.
 * User: USER
 * Date: 11/13/18
 * Time: 2:04 PM
 */
class ContentExport {
  public $logger ;
  public function __construct() {
    $this->logger = \Drupal::logger('content_export');
  }
  function load_node_config_list($bundle,$ranges_nid=[]){
    $items = [];
    $config = \Drupal::config('content_export.contentexportsetting');
    $themes_str = $config->get('path_export_content_folder');
    if($themes_str){
      if(empty($ranges_nid)){
        $items = $this->readDirectory($themes_str."/node/".$bundle);
        foreach ($items as $key => $file){
          if(file_exists($file)){
            $items[$key] = file_get_contents($file,FILE_USE_INCLUDE_PATH);
          }else{
            $this->logger->error('File  not find exist : '.$file);
          }
        }
      }else{
        for($i=$ranges_nid[0];$i < $ranges_nid[0]+1; $i++){
          $file = DRUPAL_ROOT.'/'.$themes_str."/node/".$bundle."/".$i.".yml";
          if(file_exists($file)){
            $items[$i] = file_get_contents($file,FILE_USE_INCLUDE_PATH);
          }else{
            $this->logger->error('File  not find exist : '.$file);
          }
        }
      }
    }else{
      $this->logger->error('Path directory empty ');
    }
    return $items ;
  }
  function readDirectory($directory){
    $path_file =[];
    if(is_dir(DRUPAL_ROOT.$directory)){
      $it = scandir(DRUPAL_ROOT.$directory);
      if(!empty($it)){
        foreach ($it as $fileinfo) {
          if ($fileinfo && strpos($fileinfo, '.yml') !== false) {
            $file =  DRUPAL_ROOT.$directory."/".$fileinfo ;
            if(file_exists($file)){
              $path_file[] = DRUPAL_ROOT.$directory."/".$fileinfo ;
            }
          }
        }
      }
    }
    return $path_file ;
  }

   function load_node_list($bundle,$ranges_nid=[]){
     $db = \Drupal::database();
     $query = $db->select('node_field_data','n');
     $query->fields('n', ['nid']);
     $query->condition('n.type',$bundle);
     if(!empty($ranges_nid)){
     $query->condition('n.nid',$ranges_nid,'BETWEEN');
     }
     return $query->execute()->fetchAllAssoc('nid');
   }
   function importContent($item){
     $parsed = new Parser();
     $node_object = $parsed->parse($item, SymfonyYaml::PARSE_OBJECT);
     if(is_object($node_object)){
       return $this->saving($node_object);
     }else{
       $this->logger->error('Failed to save item');
     }

   }
   function saving($node_clone){
     $node_list = \Drupal::entityTypeManager()
       ->getStorage('node')
       ->loadByProperties([
         'nid'=>$node_clone->id(),
         'title'=> $node_clone->title->value
       ]);
     if(!empty($node_list)){
        return $node_clone->save();
     }else{
        $node_clone->nid = NULL;
        $node_clone->vid = NULL;
        $node_clone->tnid = NULL;
        $node_clone->log = NULL;
        // Also handle modules that attach a UUID to the node.
        $node_clone->uuid = \Drupal::service('uuid')->generate();
        $node_clone->vuuid = NULL;
        // Anyonmymous users don't have a name.
        $node_clone->created = time();
        $node_clone->path = NULL;
        $node_clone->uid = 0;
        return $node_clone->save();
     }
   }
   function exportContent($nid){
     $node = \Drupal::entityTypeManager()
       ->getStorage('node')->load($nid);
     if($node instanceof Node){
       $yaml_content = $this->parserYAMLObject($node);
       $config = \Drupal::config('content_export.contentexportsetting');
       $themes_str = $config->get('path_export_content_folder');
       if($themes_str){
         $themes_str = DRUPAL_ROOT.'/'.$themes_str;
         return $this->generateFile($themes_str.'/node/'.$node->getType(),$node->id(),$yaml_content);
       }else{
         $this->logger->error('Path directory empty ');
         return false;
       }
     }
     return false;

   }
   function parserYAMLObject($node){
     $yaml = new Dumper(2);
     return $yaml->dump($node, PHP_INT_MAX, 0, SymfonyYaml::DUMP_OBJECT);
   }
    function generateFile($directory,$filename,$content){
      $fileSystem = \Drupal::service('file_system');
      if (!is_dir($directory)) {
        if ($fileSystem->mkdir($directory,0777, TRUE) === FALSE) {
          $this->logger->error('Failed to create directory '.$directory);
          return false ;
        }
      }
      if (file_put_contents($directory . '/' . $filename.'.yml', $content) === FALSE) {
        $this->logger->error('Failed to write file '.$filename);
        return false;
      }
      if(@chmod($directory . '/' . $filename.'.html.twig',0777)){
        $this->logger->error('Failed to change permission file '.$filename);
      }
      return true;
    }
}